import { parse, print, types } from 'recast';

var ID = '___id';
var HOT = '___hot';
var RENDERER = '___renderer';
var b = types.builders;
function index (source) {
    var ast = parse(source);
    var imports = ast.program.body.filter(function (item) { return item.type === 'ImportDeclaration'; });
    var exportFuncBlocks = ast.program.body.filter(function (item) { return item.type === 'ExportNamedDeclaration'; });
    var hmrFunctions = exportFuncBlocks.filter(function (item) { return item.comments && item.comments.some(function (comment) { return comment.type === 'Line' && comment.value.match('@hmr'); }); });
    var hmrFunctionNames = hmrFunctions.map(function (item) { return item.declaration.id.name; });
    if (hmrFunctions.length === 0)
        return source;
    var importApi = b.importDeclaration([
        b.importSpecifier(b.identifier('hot'), b.identifier(HOT)),
        b.importSpecifier(b.identifier('rerender'), b.identifier(RENDERER))
    ], b.stringLiteral('easyhard-api'));
    var idVar = b.variableDeclaration('var', [
        b.variableDeclarator(b.identifier(ID), b.stringLiteral(this.resourcePath))
    ]);
    ast.program.body.splice(imports.length, 0, idVar);
    ast.program.body.splice(0, 0, importApi);
    hmrFunctions.forEach(function (item) {
        var funcDeclaration = item.declaration;
        var funcNameIdentifier = funcDeclaration.id;
        item.declaration = b.variableDeclaration('var', [
            b.variableDeclarator(funcNameIdentifier, b.callExpression(b.identifier(HOT), [
                b.literal(funcNameIdentifier.name),
                b.identifier(ID),
                b.functionExpression(null, funcDeclaration.params, funcDeclaration.body)
            ]))
        ]);
    });
    ast.program.body.push(parse("if (module.hot) {\n    module.hot.accept();\n    " + RENDERER + "({ " + hmrFunctionNames.join(', ') + " }, " + ID + ");\n  }").program.body[0]);
    return print(ast).code;
}

export default index;
